/*  Screem:  screem-preview.c
 *
 *  The html preview widget
 *
 *  Copyright (C) 2001 David A Knight
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  For contact information with the author of this source code please see
 *  the AUTHORS file.  If there is no AUTHORS file present then check the
 *  about box under the help menu for a contact address
 */

#include <config.h>

#include <string.h>

#include <libgnomevfs/gnome-vfs-uri.h>
#include <libgnomevfs/gnome-vfs-ops.h>
#include <libgnomevfs/gnome-vfs-async-ops.h>

#include <libgnome/gnome-i18n.h>

#include <libgtkhtml/gtkhtml.h>

#ifdef SCREEM_MOZ
#include <mozilla/gtkmozembed.h>
#endif

#include "screem-view.h"

#include "screem-preview.h"
#include "screem-window.h"

static void screem_preview_display_view( ScreemView *view );
static void screem_preview_page( ScreemPreview *preview, ScreemPage *page,
				 gboolean online );

static void url_requested( HtmlDocument *document, const gchar *url,
			   HtmlStream *stream, ScreemPreview *preview );
static void url_clicked(  HtmlDocument *document, const gchar *url,
			  ScreemPreview *preview );

static void screem_preview_class_init( ScreemPreviewClass *klass );
static void screem_preview_init( ScreemPreview *preview );
static void screem_preview_finalize( GObject *preview );
static void screem_preview_window_set( ScreemView *view );
static void screem_preview_size_request( GtkWidget *widget, GtkRequisition *req );
static void screem_preview_realize( GtkWidget *widget );
static void screem_preview_show( GtkWidget *widget );


#ifndef SCREEM_MOZ
static void screem_preview_async_cb( GnomeVFSAsyncHandle *handle,
				     GnomeVFSResult result,
				     gpointer data );
void screem_preview_async_read_cb( GnomeVFSAsyncHandle *handle,
				   GnomeVFSResult result,
				   gpointer buffer,
				   GnomeVFSFileSize bytes_requested,
				   GnomeVFSFileSize bytes_read,
				   gpointer data );
static void screem_preview_async_close_cb( GnomeVFSAsyncHandle *handle,
					   GnomeVFSResult result,
					   gpointer data );


static void screem_preview_remote_async_cb( GnomeVFSAsyncHandle *handle,
					    GnomeVFSResult result,
					    gpointer data );


static void screem_preview_remote_async_read_cb( GnomeVFSAsyncHandle *handle,
						 GnomeVFSResult result,
						 gpointer buffer,
						 GnomeVFSFileSize bytes_requested,
						 GnomeVFSFileSize bytes_read,
						 gpointer data );
#endif


static gboolean screem_preview_cancel_all( gpointer value,
					   ScreemPreview *preview );

struct ScreemPreviewPrivate {
	GtkWidget *sw;
	GtkWidget *html;

	HtmlDocument *document;

	GList *streams;
	GMutex *streams_lock;

	GList *handles;

	gboolean finalising;
};

ScreemPreview *screem_preview_new()
{
	ScreemPreview *preview;
	GType type;

	type = screem_preview_get_type();

	preview = SCREEM_PREVIEW( g_object_new( type, NULL ) );

	return preview;
}

ScreemPreview *screem_preview_new_with_window( ScreemWindow *window )
{

	ScreemPreview *preview;
	GType type;

	type = screem_preview_get_type();

	preview = SCREEM_PREVIEW( g_object_new( type, "window", window,
						NULL ) );

	return preview;
}

static void screem_preview_display_view( ScreemView *view )
{
	ScreemSite *site;
	ScreemPage *page;
	gboolean online;

	const gchar *baseuri;
	const gchar *pagepath;

	gboolean dynamic;

	g_object_get( G_OBJECT( view ),
		      "site", &site,
		      "page", &page,
		      "online", &online,
		      NULL );

	if( site && ! screem_site_get_fake_flag( site ) ) {
		baseuri = screem_site_get_http_url( site );
	} else {
		baseuri = NULL;
	}
	if( page ) {
		dynamic = screem_page_is_dynamic( page );
		pagepath = screem_page_get_pathname( page );
	} else {
		dynamic = FALSE;
		pagepath = NULL;
	}
	if( pagepath && strlen( pagepath ) && baseuri && 
	    ( ( ! dynamic && 
		screem_site_get_preview_static( site ) ) ||
	      ( dynamic && screem_site_get_preview_dynamic( site ) ) ) ) {
		gchar *uri;
		
		pagepath = g_basename( pagepath );
		uri = g_strconcat( "http://", baseuri, "/", pagepath, NULL );
		
		screem_preview_uri( SCREEM_PREVIEW( view ),
				    uri, online );
		
		g_free( uri );
	} else {
		screem_preview_page( SCREEM_PREVIEW( view ),
				     page, online );
	}
}

static void screem_preview_page( ScreemPreview *preview, ScreemPage *page,
				 gboolean online )
{
	g_return_if_fail( SCREEM_IS_PREVIEW( preview ) );

#ifndef SCREEM_MOZ
	g_mutex_lock( preview->private->streams_lock );
	g_list_free( preview->private->streams );
	preview->private->streams = NULL;
	g_mutex_unlock( preview->private->streams_lock );
#endif

	if( page ) {
		const gchar *pathname;
		const gchar *mime_type;
		gchar *data;
		gchar *dirname;
		gchar *base_uri;
		GnomeVFSURI *uri;

		pathname = screem_page_get_pathname( page );
		mime_type = screem_page_get_mime_type( page );

		if( ! mime_type || 
		    ( strcmp( "text/html", mime_type ) &&
		      strcmp( "application/x-php", mime_type ) ) ) {
			data = g_strconcat( "<html><body style=\"background: black; color: white; text-align: center\">", _( "Not an HTML document " ),
					    "</body></html>", NULL );
		} else {
			data = screem_page_get_data( page );
		}
		if( pathname ) {
			uri = gnome_vfs_uri_new( pathname );
			if( uri ) {
				dirname = gnome_vfs_uri_extract_dirname( uri );
				base_uri = 
					g_strconcat( gnome_vfs_uri_get_scheme( uri ),
						     "://",
						     dirname, "/", NULL );
			} else {
				dirname = NULL;
				base_uri = g_strdup( "file:///" );
			}
		} else {
			base_uri = NULL;
			uri = NULL;
			dirname = NULL;
		}
		screem_preview_display( preview, data, base_uri, online );
		
		if( uri ) {
			gnome_vfs_uri_unref( uri );
		}
		if( dirname ) {
			g_free( dirname );
		}
		if( base_uri ) {
			g_free( base_uri );
		}
		if( data ) {
			g_free( data );
		}
	} else {
		/* can't pass an empty string as HtmlStream gets upset */
		screem_preview_display( preview, NULL, NULL, online );
	}
}


void screem_preview_display( ScreemPreview *preview, const gchar *text,
			     const gchar *base_uri, gboolean online )
{
	g_return_if_fail( SCREEM_IS_PREVIEW( preview ) );

	if( ! text || ( strlen( text ) == 0 )) {
		text ="<html><body></body></html>";
	}
	if( ! base_uri ) {
		base_uri = "file:///";
	}

	g_object_set_data( G_OBJECT( preview ), "base_uri",
			   (gpointer)base_uri );

#ifndef SCREEM_MOZ
	{
		HtmlDocument *document;

		document = preview->private->document;
		html_document_clear( document );

		g_object_set_data( G_OBJECT( document ), "online",
				   GINT_TO_POINTER( online ) );

		html_document_open_stream( document, "text/html" );
		html_document_write_stream( document, text, strlen( text ) );
		html_document_close_stream( document );
	}
#else
	gtk_moz_embed_render_data( GTK_MOZ_EMBED( preview->private->html ), 
				   text, strlen( text ),
				   base_uri, "text/html" );
#endif
}

void screem_preview_uri( ScreemPreview *preview, const gchar *uri, 
			 gboolean online )
{
#ifndef SCREEM_MOZ
	HtmlDocument *document;
	GnomeVFSAsyncHandle *handle;
	gchar *baseuri;

	document = preview->private->document;
	html_document_clear( document );

	/* FIXME: MEM LEAK */
	baseuri = g_dirname( uri );

	html_document_open_stream( document, "text/html" );

	g_object_set_data( G_OBJECT( preview ), "base_uri",
			   (gpointer)baseuri );
	g_object_set_data( G_OBJECT( document ), "online",
				   GINT_TO_POINTER( online ) );

	if( online ) {
		g_mutex_lock( preview->private->streams_lock );
		gnome_vfs_async_open( &handle, uri, GNOME_VFS_OPEN_READ,
				      -10, 
				      (GnomeVFSAsyncOpenCallback)
				      screem_preview_remote_async_cb,
				      preview );
		preview->private->handles = 
			g_list_append( preview->private->handles,
				       handle );
		g_mutex_unlock( preview->private->streams_lock );
	}

#endif
}

void screem_preview_print( ScreemPreview *preview, gboolean prview )
{

}

#ifndef SCREEM_MOZ
static void url_requested( HtmlDocument *document, const gchar *url,
			   HtmlStream *stream, ScreemPreview *preview )
{
	GnomeVFSAsyncHandle *handle;
	GnomeVFSURI *uri;
	gboolean islocal;
	const gchar *base_uri;
	gchar *fullurl;

	gchar *base;
	gboolean online;

	online = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( document ), 
						     "online" ) );

	if( ! strlen( url ) ) {
		return;
	}

	g_mutex_lock( preview->private->streams_lock );

	preview->private->streams = g_list_append( preview->private->streams,
						   stream );
	g_object_set_data( G_OBJECT( stream ), "preview", preview );

	g_mutex_unlock( preview->private->streams_lock );

	base_uri = g_object_get_data( G_OBJECT( preview ), "base_uri" );
	if( ! base_uri ) {
		base = g_strdup( "file://" );
	} else if( base_uri[ strlen( base_uri ) - 1 ] != '/' ) {
		base = g_strconcat( base_uri, "/", NULL );
	} else {
		base = g_strdup( base_uri );
	}
	fullurl = gnome_vfs_uri_make_full_from_relative( base, url );
	g_free( base );

	uri = gnome_vfs_uri_new( fullurl );
	islocal = gnome_vfs_uri_is_local( uri );
	gnome_vfs_uri_unref( uri );
	
	if( islocal || online ) {
		g_mutex_lock( preview->private->streams_lock );
		gnome_vfs_async_open( &handle, fullurl, GNOME_VFS_OPEN_READ,
				      -10, 
				      (GnomeVFSAsyncOpenCallback)
				      screem_preview_async_cb,
				      stream );
		preview->private->handles =
			g_list_append( preview->private->handles,
				       handle );
		g_mutex_unlock( preview->private->streams_lock );
	}
	g_free( fullurl );
}

static void screem_preview_async_cb( GnomeVFSAsyncHandle *handle,
				     GnomeVFSResult result,
				     gpointer data )
{
	gchar *buffer;
	HtmlStream *stream;
	ScreemPreview *preview;
	GList *list = NULL;

	stream = HTML_STREAM( data );
	
	preview = SCREEM_PREVIEW( g_object_get_data( G_OBJECT(stream),
						     "preview" ) );
	
	g_mutex_lock( preview->private->streams_lock );
	
	/* if stream is still in preview->private->streams we may continue */
	list = g_list_find( preview->private->streams, data );
		
	g_mutex_unlock( preview->private->streams_lock );

	if( list ) {
		switch( result ) {
		case GNOME_VFS_OK:
			/* exists */
			buffer = g_new0( gchar, 1024 );
			gnome_vfs_async_read( handle, buffer, 1024,
					      screem_preview_async_read_cb,
					      data );
			break;
		default:
			html_stream_close( stream );
			g_mutex_lock( preview->private->streams_lock );
			if( g_list_find( preview->private->streams,stream ) ) {
				preview->private->streams = 
					g_list_remove( preview->private->streams,
						       stream );
			}
			preview->private->handles = 
				g_list_remove( preview->private->handles,
					       handle );
			g_mutex_unlock( preview->private->streams_lock );
			break;
		}
	} else {
		html_stream_destroy( stream );
		g_mutex_lock( preview->private->streams_lock );
		if( ! preview->private->finalising ) {
			gnome_vfs_async_close( handle,
					       screem_preview_async_close_cb,
					       preview );
		}
		g_mutex_unlock( preview->private->streams_lock );
	}
}

void screem_preview_async_read_cb( GnomeVFSAsyncHandle *handle,
				   GnomeVFSResult result,
				   gpointer buffer,
				   GnomeVFSFileSize bytes_requested,
				   GnomeVFSFileSize bytes_read,
				   gpointer data )
{
	HtmlStream *stream;
	ScreemPreview *preview;
	GList *list = NULL;

	stream = HTML_STREAM( data );
	
	preview = SCREEM_PREVIEW( g_object_get_data( G_OBJECT(stream),
						     "preview" ) );

	g_mutex_lock( preview->private->streams_lock );
	
	/* if stream is still in preview->private->streams we may continue */
	list = g_list_find( preview->private->streams, data );

	g_mutex_unlock( preview->private->streams_lock );
	
	if( list ) {
		switch( result ) {
		case GNOME_VFS_OK:
			if( bytes_read > 0 ) {
				html_stream_write( stream, buffer, bytes_read);
			}
			g_mutex_lock( preview->private->streams_lock );
			if( ! preview->private->finalising ) {
				gnome_vfs_async_read( handle, buffer, 1024,
						      screem_preview_async_read_cb,
						      data );
			}
			g_mutex_unlock( preview->private->streams_lock );
			break;
		default:
			html_stream_close( stream );
			g_mutex_lock( preview->private->streams_lock );
			if( g_list_find( preview->private->streams,stream ) ) {
				preview->private->streams = 
					g_list_remove( preview->private->streams,
						       stream );
			}
			if( ! preview->private->finalising ) {
				gnome_vfs_async_close( handle, 
						       screem_preview_async_close_cb, 
						       preview );
			}
			g_mutex_unlock( preview->private->streams_lock );

			break;
		}
	} else {
		html_stream_destroy( stream );
		g_mutex_lock( preview->private->streams_lock );
		if( ! preview->private->finalising ) {
			gnome_vfs_async_close( handle,
					       screem_preview_async_close_cb,
					       preview );
		}
		g_mutex_unlock( preview->private->streams_lock );
	}
}

static void screem_preview_async_close_cb( GnomeVFSAsyncHandle *handle,
					   GnomeVFSResult result,
					   gpointer data )
{
	ScreemPreview *preview;

	preview = SCREEM_PREVIEW( data );

	g_mutex_lock( preview->private->streams_lock );
	preview->private->handles = g_list_remove( preview->private->handles,
						   handle );
	g_mutex_unlock( preview->private->streams_lock );
}

static void url_clicked( HtmlDocument *document, const gchar *url, 
			 ScreemPreview *preview )
{
#if 0
	const gchar *site_url;
	ScreemPage *page;
	GnomeVFSURI *vfs_uri;

	gchar *protocol;
	gchar *address;
	gchar *port;
	gchar *path;
	gchar *full;

	vfs_uri = gnome_vfs_uri_new( url );
	protocol = gnome_vfs_uri_get_scheme( vfs_uri );
	address = gnome_vfs_uri_get_host_name( vfs_uri );
	port = g_strdup_printf( "%i", gnome_vfs_uri_get_host_port( vfs_uri ) );
	path = gnome_vfs_uri_get_path( vfs_uri );

	if( ! help ) {
		site_url = screem_site_get_http_url( current_site );
		if( address && site_url && strcmp( address, site_url ) ) {
			/* its not a link in this site */
			g_free( path );
			g_free( protocol );
			g_free( address );
			g_free( port );
			return;
		}
	
		/* convert path to a complete pathname */
		full = relative_to_full( path );
		
		page = screem_site_locate_page( current_site, full );
		if( page ) {
			/* switch to it! */
			screem_page_insert( page );
		}
	}

	g_free( full );
	g_free( protocol );
	g_free( address );
	g_free( port );
	g_free( path );
#endif
}

static void screem_preview_remote_async_cb( GnomeVFSAsyncHandle *handle,
					    GnomeVFSResult result,
					    gpointer data )
{
	ScreemPreview *preview;
	gchar *buffer;

	preview = SCREEM_PREVIEW( data );

	g_mutex_lock( preview->private->streams_lock );
	if( result == GNOME_VFS_OK &&
	    ( ! g_list_find( preview->private->handles, handle ) ) &&
	    ( ! preview->private->finalising ) ) {
		gnome_vfs_async_close( handle, 
				       screem_preview_async_close_cb, 
				       preview );
		return;
	}
	g_mutex_unlock( preview->private->streams_lock );

	switch( result ) {
	case GNOME_VFS_OK:
		/* exists */
		buffer = g_new0( gchar, 1024 );
		gnome_vfs_async_read( handle, buffer, 1024,
				      screem_preview_remote_async_read_cb,
				      data );
		break;
	default:
		g_mutex_lock( preview->private->streams_lock );
		preview->private->handles = 
			g_list_remove( preview->private->handles,
				       handle );
		g_mutex_unlock( preview->private->streams_lock );
		break;
	}
}

static void screem_preview_remote_async_read_cb( GnomeVFSAsyncHandle *handle,
						 GnomeVFSResult result,
						 gpointer buffer,
						 GnomeVFSFileSize bytes_requested,
						 GnomeVFSFileSize bytes_read,
						 gpointer data )
{
	ScreemPreview *preview;

	preview = SCREEM_PREVIEW( data );

	g_mutex_lock( preview->private->streams_lock );
	if( ( ! g_list_find( preview->private->handles, handle ) ) &&
	    ! preview->private->finalising ) {
		gnome_vfs_async_close( handle, 
				       screem_preview_async_close_cb, 
				       preview );
		return;
	}
	g_mutex_unlock( preview->private->streams_lock );

	switch( result ) {
	case GNOME_VFS_OK:
		if( bytes_read > 0 ) {
			html_document_write_stream( preview->private->document, 
						    buffer, bytes_read );

		}
		g_mutex_lock( preview->private->streams_lock );
		if( ! preview->private->finalising ) {
			gnome_vfs_async_read( handle, buffer, 1024,
					      screem_preview_remote_async_read_cb,
					      data );
		}
		g_mutex_unlock( preview->private->streams_lock );
		break;
	default:
		html_document_close_stream( preview->private->document );

		g_mutex_lock( preview->private->streams_lock );
		if( ! preview->private->finalising ) {
			gnome_vfs_async_close( handle, 
					       screem_preview_async_close_cb, 
					       preview );
		}
		g_mutex_unlock( preview->private->streams_lock );
		
		break;
	}
}     

#endif

/* G Object stuff */

#define PARENT_TYPE SCREEM_TYPE_VIEW

static gpointer parent_class;

static void screem_preview_class_init( ScreemPreviewClass *klass )
{
	GObjectClass *object_class;
	GtkWidgetClass *widget_class;

	object_class = G_OBJECT_CLASS( klass );
	widget_class = (GtkWidgetClass *)klass;
	parent_class = g_type_class_peek_parent( klass );

	object_class->finalize = screem_preview_finalize;

	widget_class->show = screem_preview_show;
	widget_class->realize = screem_preview_realize;
	widget_class->size_request = screem_preview_size_request;
}

#ifndef SCREEM_MOZ
static void screem_preview_init( ScreemPreview *preview )
{
	GtkWidget *html;
	GtkWidget *sw;

	ScreemPreviewPrivate *private;
	HtmlDocument *document;

	SCREEM_VIEW( preview )->window_set = screem_preview_window_set;
	SCREEM_VIEW( preview )->display = screem_preview_display_view;
	SCREEM_VIEW( preview )->print = screem_preview_print;

	private = preview->private = g_new0( ScreemPreviewPrivate, 1 );

	document = private->document = html_document_new();
	g_signal_connect_data( G_OBJECT( document ), "request_url",
			       G_CALLBACK( url_requested ), preview,
			       NULL, 0 );
	g_signal_connect_data( G_OBJECT( document ), "link_clicked",
			       G_CALLBACK( url_clicked ), preview,
			       NULL, 0 );

	private->streams_lock = g_mutex_new();

	html = private->html = html_view_new();

	gtk_widget_show( private->html );

	sw = preview->private->sw = 
		gtk_scrolled_window_new( GTK_LAYOUT( html )->hadjustment,
					 GTK_LAYOUT( html )->vadjustment );
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( sw ),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC );

	gtk_container_add( GTK_CONTAINER( sw ), html );


	gtk_box_pack_start( GTK_BOX( preview ), sw, TRUE, TRUE, 0 );

        html_view_set_document( HTML_VIEW( html ), document );
}
#else
static void screem_preview_init( ScreemPreview *preview )
{
	GtkWidget *html;

        const gchar *moz5home = g_getenv( "MOZILLA_FIVE_HOME" );

	const gchar *signals[] = {
                        "link_message","js_status","location",
                        "title", "progress", "progress_all",
                        "net_state", "net_state_all", "net_start",
                        "net_stop", "new_window", "visibility",
                        "destroy_browser", "open_uri", "size_to",
                        "dom_key_down", "dom_key_press", "dom_key_up",
                        "dom_mouse_down", "dom_mouse_up", "dom_mouse_click",
                        "dom_mouse_dbl_click", "dom_mouse_over",
                        "dom_mouse_out", NULL
        };
	const gpointer cbs[] = {
                NULL, NULL, NULL, NULL,
                NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                NULL, NULL, NULL, NULL
        };
	
	int i;

	SCREEM_VIEW( preview )->window_set = screem_preview_window_set;
	SCREEM_VIEW( preview )->display = screem_preview_display_view;
	SCREEM_VIEW( preview )->print = screem_preview_print;

        gtk_moz_embed_set_comp_path( moz5home );
        html = preview->private->html = gtk_moz_embed_new();

	for( i = 0; signals[ i ]; i ++ ) {
                g_signal_connect( G_OBJECT( html ),
				  signals[ i ],
				  G_CALLBACK( cbs[ i ] ),
				  preview );
	}

        gtk_moz_embed_set_chrome_mask( html, GTK_MOZ_EMBED_FLAG_ALLCHROME );

	gtk_widget_show( html );

	gtk_box_pack_start( GTK_BOX( preview ), preview->private->html,
			    TRUE, TRUE, 0 );
}
#endif

static void screem_preview_finalize( GObject *object )
{
	ScreemPreview *preview;

	preview = SCREEM_PREVIEW( object );

#ifndef SCREEM_MOZ
	g_mutex_lock( preview->private->streams_lock );

	preview->private->finalising = TRUE;

	g_list_free( preview->private->streams );
	preview->private->streams = NULL;

	g_list_foreach( preview->private->handles,
			(GFunc)screem_preview_cancel_all,
			preview );
	g_list_free( preview->private->handles );
	preview->private->handles = NULL;

	g_mutex_unlock( preview->private->streams_lock );
#endif

	for( ;; ) {
		g_mutex_lock( preview->private->streams_lock );
		if( ! preview->private->handles ) {
			break;
		}
		g_mutex_unlock( preview->private->streams_lock );
		sleep( 1 );
	}
	g_mutex_unlock( preview->private->streams_lock );

	g_mutex_free( preview->private->streams_lock );

	G_OBJECT_CLASS( parent_class )->finalize( object );
}

static gboolean screem_preview_cancel_all( gpointer value,
					   ScreemPreview *preview )
{
	gnome_vfs_async_cancel( value );
	
	return FALSE;
}

static void screem_preview_window_set( ScreemView *view )
{

}

static void screem_preview_size_request( GtkWidget *widget, GtkRequisition *req )
{
	GTK_WIDGET_CLASS( parent_class )->size_request( widget, req );
}

static void screem_preview_realize( GtkWidget *widget )
{
	GTK_WIDGET_CLASS( parent_class )->realize( widget );
}

static void screem_preview_show( GtkWidget *widget )
{
	ScreemPreview *preview;

	preview = SCREEM_PREVIEW( widget );

	GTK_WIDGET_CLASS( parent_class )->show( widget );

	gtk_widget_show( preview->private->html );

	if( preview->private->sw ) {
		gtk_widget_show( preview->private->sw );
	}
}

GType screem_preview_get_type()
{
	static GType type = 0;

	if( ! type ) {
		static const GTypeInfo info = {
			sizeof( ScreemPreviewClass ),
			NULL, /* base init */
			NULL, /* base finalise */
			(GClassInitFunc)screem_preview_class_init,
			NULL, /* class finalise */
			NULL, /* class data */
			sizeof( ScreemPreview ),
			0, /* n_preallocs */
			(GInstanceInitFunc)screem_preview_init
		};
		
		type = g_type_register_static( PARENT_TYPE,
					       "ScreemPreview",
					       &info, 0 );
	}

	return type;
}
